@@ -10,7 +10,7 @@ from noseprogressive.wrapping import cmdloop, set_trace, StreamWrapper
 
 
 class ProgressivePlugin(Plugin):
-    """Nose plugin which prioritizes the important information"""
+    """A nose plugin which has a progress bar and formats tracebacks for humans"""
     name = 'progressive'
     _totalTests = 0
     score = 10000  # Grab stdout and stderr before the capture plugin.
@@ -49,8 +49,8 @@ class ProgressivePlugin(Plugin):
         pdb.Pdb.cmdloop = cmdloop
 
         # nosetests changes directories to the tests dir when run from a
-        # distribution dir, so save the original cwd.
-        self._cwd = getcwd()
+        # distribution dir, so save the original cwd for relativizing paths.
+        self._cwd = '' if self.conf.options.absolute_paths else getcwd()
 
     def finalize(self, result):
         """Put monkeypatches back as we found them."""
@@ -61,17 +61,32 @@ class ProgressivePlugin(Plugin):
 
     def options(self, parser, env):
         super(ProgressivePlugin, self).options(parser, env)
+        parser.add_option('--progressive-function-color',
+                          type='int',
+                          dest='function_color',
+                          default=env.get('NOSE_PROGRESSIVE_FUNCTION_COLOR', 12),
+                          help='Color of function names in tracebacks. An '
+                               'ANSI color expressed as a number 0-15.')
+        parser.add_option('--progressive-dim-color',
+                          type='int',
+                          dest='dim_color',
+                          default=env.get('NOSE_PROGRESSIVE_DIM_COLOR', 8),
+                          help='Color of de-emphasized text (like editor '
+                               'shortcuts) in tracebacks. An ANSI color '
+                               'expressed as a number 0-15.')
         parser.add_option('--progressive-advisories',
                           action='store_true',
-                          dest='showAdvisories',
+                          dest='show_advisories',
                           default=env.get('NOSE_PROGRESSIVE_ADVISORIES', False),
                           help='Show skips and deprecation exceptions in '
                                'addition to failures and errors.')
-
-    def configure(self, options, config):
-        super(ProgressivePlugin, self).configure(options, config)
-        if self.can_configure:
-            self._showAdvisories = options.showAdvisories
+        parser.add_option('--progressive-abs',
+                          action='store_true',
+                          dest='absolute_paths',
+                          default=env.get('NOSE_PROGRESSIVE_ABSOLUTE_PATHS', False),
+                          help='Display paths in traceback as absolute, '
+                               'rather than relative to the current working '
+                               'directory.')
 
     def prepareTestLoader(self, loader):
         """Insert ourselves into loader calls to count tests.
@@ -107,7 +122,6 @@ class ProgressivePlugin(Plugin):
         return ProgressiveRunner(self._cwd,
                                  self._totalTests,
                                  runner.stream,
-                                 self._showAdvisories,
                                  verbosity=self.conf.verbosity,
                                  config=self.conf)  # So we don't get a default
                                                     # NoPlugins manager
