@@ -1,15 +1,10 @@
-from collections import defaultdict
-from curses import tigetstr, setupterm, tparm
-import os
-from os import isatty
-from traceback import format_list, extract_tb, format_exception_only
-
 from nose.result import TextTestResult
-from nose.util import src, isclass
+from nose.util import isclass
 
 from noseprogressive.bar import ProgressBar
-from noseprogressive.utils import (nose_selector, human_path, frame_of_test,
-                                   test_address)
+from noseprogressive.terminal import Terminal
+from noseprogressive.tracebacks import format_traceback, extract_relevant_tb
+from noseprogressive.utils import nose_selector, index_of_test_frame
 
 
 class ProgressiveResult(TextTestResult):
@@ -20,41 +15,19 @@ class ProgressiveResult(TextTestResult):
     stderr/out wrapping.
 
     """
-    def __init__(self, cwd, totalTests, stream, showAdvisories, config=None):
+    def __init__(self, cwd, totalTests, stream, config=None):
         super(ProgressiveResult, self).__init__(stream, None, 0, config=config)
         self._cwd = cwd
-        self._codes = self._terminalCodes(stream)
+        self._term = Terminal(stream=stream)
 
         # 1 in case test counting failed and returned 0
-        self.bar = ProgressBar(stream, totalTests or 1, self._codes)
+        self.bar = ProgressBar(stream, totalTests or 1, self._term)
 
         # Declare errorclass-savviness so the errorclass plugin doesn't
         # monkeypatch half my methods away:
         self.errorClasses = {}
 
-        self._showAdvisories = showAdvisories
-
-    def _terminalCodes(self, stream):
-        """Return a hash of termcap codes and values for the terminal `stream`.
-
-        If `stream` is not a terminal, return empty values so you can pipe this
-        to a file without making a mess of it. (I'm not sure why you would want
-        to do that; this is mostly in place to make the Progressive's test
-        suite run.)
-
-        """
-        capabilities = ['bold', 'sc', 'rc', 'sgr0', 'el']
-        if hasattr(stream, 'fileno') and isatty(stream.fileno()):
-            # Explicit args make setupterm() work even when -s is passed:
-            setupterm(None, stream.fileno())  # so things like tigetstr() work
-            codes = dict((x, tigetstr(x)) for x in capabilities)
-            cup = tigetstr('cup')
-            codes['cup'] = lambda line, column: tparm(cup, line, column)
-        else:
-            # If you're crazy enough to pipe this to a file or something, don't
-            # output terminal codes:
-            codes = defaultdict(lambda: '', cup=lambda line, column: '')
-        return codes
+        self._options = config.options
 
     def startTest(self, test):
         """Update the progress bar."""
@@ -69,53 +42,46 @@ class ProgressiveResult(TextTestResult):
         test -- the test that precipitated this call
 
         """
-        if isFailure or self._showAdvisories:
-            # Don't bind third item to a local var; that can create circular
-            # refs which are expensive to collect. See the sys.exc_info() docs.
-            exception_type, exception_value = err[:2]
-            extracted_tb = extract_tb(err[2])
-            formatted_traceback = ''.join(format_list(extracted_tb))
-            # TODO: Canonicalize the path to remove /kitsune/../kitsune
-            # nonsense. Don't relativize, though, as that hurts the ability to
-            # paste into running editors.
+        if isFailure or self._options.show_advisories:
             writeln = self.stream.writeln
             write = self.stream.write
             with self.bar.dodging():
-                writeln('\n' + (self._codes['bold'] if isFailure else '') +
-                        '%s: %s' % (kind, nose_selector(test)))
+                writeln('\n' +
+                        (self._term.bold if isFailure else '') +
+                        '%s: %s' % (kind, nose_selector(test)) +
+                        (self._term.normal if isFailure else ''))  # end bold
 
                 if isFailure:  # Then show traceback
-                    # File name and line num in a format vi can take:
-                    try:
-                        address = test_address(test)
-                    except TypeError:
-                        # Explodes if the function passed to @with_setup
-                        # applied to a test generator has an error.
-                        address = None
-                    if address:  # None if no such callable found. No sense
-                                 # trying to find the test frame if there's no
-                                 # such thing.
-                        file, line = frame_of_test(address,
-                                                   exception_type,
-                                                   exception_value,
-                                                   extracted_tb)[:2]
-                        writeln(' ' * len(kind) + '  %s +%s %s' %
-                                (os.environ.get('EDITOR', 'vi'), line,
-                                 human_path(src(file), self._cwd)))
+                    # Don't bind third item to a local var; that can create
+                    # circular refs which are expensive to collect. See the
+                    # sys.exc_info() docs.
+                    exception_type, exception_value = err[:2]
+                    extracted_tb = extract_relevant_tb(
+                        err[2],
+                        exception_type,
+                        exception_type is test.failureException)
+                    test_frame_index = index_of_test_frame(
+                        extracted_tb,
+                        exception_type,
+                        exception_value,
+                        test)
+                    if test_frame_index:
+                        # We have a good guess at which frame is the test, so
+                        # trim everything until that. We don't care to see test
+                        # framework frames.
+                        extracted_tb = extracted_tb[test_frame_index:]
 
-                    write(self._codes['sgr0'])  # end bold
-
-                    # Traceback:
-                    # TODO: Think about using self._exc_info_to_string, which
-                    # does some pretty whizzy skipping of unittest frames.
+                    formatted_traceback = ''.join(
+                        format_traceback(
+                            extracted_tb,
+                            exception_type,
+                            exception_value,
+                            self._cwd,
+                            self._term,
+                            self._options.function_color,
+                            self._options.dim_color))
                     write(formatted_traceback)
 
-                    # Exception:
-                    write(''.join(format_exception_only(exception_type,
-                                                        exception_value)))
-                else:
-                    write(self._codes['sgr0'])  # end bold
-
     def addError(self, test, err):
         exc, val, tb = err
         # We don't read this, but some other plugin might conceivably expect it
@@ -160,7 +126,7 @@ class ProgressiveResult(TextTestResult):
             # being Englishlike:
             ret = '%s %s%s' % (number, type, 's' if number != 1 else '')
             if isFailure and number:
-                ret = self._codes['bold'] + ret + self._codes['sgr0']
+                ret = self._term.bold + ret + self._term.normal
             return ret
 
         # Summarize the special cases:
